searchData={"items":[{"type":"module","title":"Without","doc":"`Without` is a tiny module to ease the usage of result tuple.\n\nUsing `case` once is convenient, using it more than twice, makes the code less readable, consider this snippet\n\n```\ncase find_user(email) do\n  {:ok, user} -> \n    case find_friends(user) do\n      {:ok, friends} -> \"#{user} is friends of #{Enum.join(\",\", friends)}\"\n      {:error, :friends_not_found} -> \"#{user} doesn't have any friends\"\n    end\n  {:error, :user_not_found} -> \"user not found\"\nend\n```\n\nYou might ask maybe `with` could help on that?\n```\nwith {:ok, user} <- find_user(email),\n     {:ok, friends} <- find_friends(user) do\n    \"#{user} is friends of #{Enum.join(\",\", friends)}\"\n\nelse\n  {:error, :user_not_found} -> \"user not found\"\n  {:error, :friends_not_found} -> \"#{user} doesn't have any friends\"\nend\n```\nBut the issue here is that variable `user` is not available in the last else case!\n\n\nNow that you feel the pain, let me introduce you to `Without`!\n```\nemail\n|> Without.fmap_ok(&find_user/1, assign: :user)\n|> Without.fmap_ok(&find_friends/1)\n|> Without.fmap_error(fn error, assigns ->\n  case error do\n  :user_not_found -> \"user not found\"\n  :friends_not_found, assigns -> \"#{assigns[:user]} doesn't have any friends\" \nend)\n|> Without.fresult\n```\n\nIf you are a functional programming aficionado, it might resemble monadic error handling.\n\nOne more thing! `Without` is a lazy operator. It means your pipeline won't be executed until `Without.fresult/1` is called.\nIt gives you the flexibility to build your pipeline by passing it around different module/functions and execute them on the last step.\n\n> WORD OF CAUTION: Use the following snippet with cautious, I haven't used this technique and might make your code complex and unreadable\n\n\n```\ndef fetch_user(user, %Without{} = without) do\n  without\n  # Fetch it from somewhere if only previous steps are :ok!\n  |> Without.fmap_ok(fn -> {:ok, user} end, assign: :user)\nend\n\ndef fetch_friends(%Without{} = without) do\n  without\n  # Fetch it from somewhere if only previous steps are :ok!\n  |> Without.fmap_ok(\n    fn _, assigns -> {:ok, [\"#{assigns[:user]}-f01\", \"#{assigns[:user]}-f01\"]} end,\n    assign: :friends\n  )\nend\n\ndef render(%Without{} = without) do\n  without\n  |> Without.fmap_ok(fn friends ->\n    conn = render(conn, friends: friends)\n    {:ok, conn}\n  end)\n  |> Without.fmap_error(fn error ->\n    Logger.error(\"failed to make external calls due to #{error}\")\n    conn = render(conn, error: error)\n    {:ok, conn}\n  end)\nend\n\ndef index(conn, _) do\n  required_external_calls = Without.finit(nil)\n  # do other things.....\n  required_external_calls = fetch_user(\"milad\", required_external_calls)\n  # do something else ...\n  required_external_calls = fetch_friends(required_external_calls)\n\n  {:ok, conn} = \n    required_external_calls\n    |> render()\n    |> Without.fresult()\n\n  conn\nend\n```","ref":"Without.html"},{"type":"function","title":"Without.finit/1","doc":"","ref":"Without.html#finit/1"},{"type":"function","title":"Without.fmap_error/2","doc":"","ref":"Without.html#fmap_error/2"},{"type":"function","title":"Without.fmap_ok/3","doc":"","ref":"Without.html#fmap_ok/3"},{"type":"function","title":"Without.fresult/1","doc":"","ref":"Without.html#fresult/1"},{"type":"type","title":"Without.error/0","doc":"","ref":"Without.html#t:error/0"},{"type":"type","title":"Without.map_func/0","doc":"","ref":"Without.html#t:map_func/0"},{"type":"type","title":"Without.ok/0","doc":"","ref":"Without.html#t:ok/0"},{"type":"type","title":"Without.options/0","doc":"","ref":"Without.html#t:options/0"},{"type":"type","title":"Without.result/0","doc":"","ref":"Without.html#t:result/0"},{"type":"type","title":"Without.t/0","doc":"","ref":"Without.html#t:t/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}